// filename ******** heap.c ************** 
// Implementations for the simple heap example   

//  This example accompanies the books
//   "Embedded Microcomputer Systems: Real Time Interfacing", 
//     Thomsen, copyright (c) 2006,
//   "Introduction to Embedded Microcomputer Systems: 
//    Motorola 6811 and 6812 Simulation", Brooks-Cole, copyright (c) 2002,
//    Jonathan W. Valvano 3/21/08

// You may use, edit, run or distribute this file 
//    as long as the above copyright notices remain
#include "heap.h"

char *FreePt; 
char Heap[SIZE*NUM];

//*************Heap_Init***************
// partition the heap into NUM blocks of SIZE bytes
// heap will leave the heap empty 
// Inputs: none
// Outputs: none
// errors: this will destroy any data previously allocated
// You should call this function exactly once at the start
void Heap_Init(void){ 
char *pt;
  FreePt = &Heap[0];
  for(pt=&Heap[0];
      pt!=&Heap[SIZE*(NUM-1)];
      pt=pt+SIZE){ 
    *(short*)pt =(short)(pt+SIZE); 
  }
  *(short*)pt = null; 
}

//*************Heap_Allocate***************
// allocate 1 block of SIZE bytes
// Inputs: none
// Outputs: pointer to block of BYTE bytes
// errors: returns NULL pointer if heap was empty
void *Heap_Allocate(void){
char *pt;
  pt = FreePt;
  if (pt != null){ 
    FreePt = (char*) *(char**)pt;
  }
  return(pt);
}

//*************Heap_Release***************
// release 1 block of SIZE bytes back to heap
// Inputs: pointer to block of BYTE bytes
// Outputs: pointer to block of BYTE bytes
// errors: it does not check to make sure the pointer is valid
void Heap_Release(void *pt){ 
char *oldFreePt;
  oldFreePt = FreePt;
  FreePt = (char*)pt;
  *(short*)pt = (short)oldFreePt; 
}
